#version 330 core

struct Material {
	sampler2D diffuseTexture;

	vec3 ambient; 
	vec3 diffuse; 
	vec3 specular;

	float shininess;
};

struct GlobalLight {
	vec3 position;

	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};
  
in vec2 TextureCoords;
in vec3 Normal;
in vec3 FragmentPosition;
in vec4 FragPosLightSpace;

out vec4 color;

uniform Material material;
uniform GlobalLight globalLight;

uniform vec3 viewPosition;

uniform sampler2D shadowMap;

uniform bool blinn;
uniform bool shadows;
uniform bool celShading;

// Function prototypes
vec3 CalcGlobalLight(GlobalLight light, vec3 normal, vec3 viewDirection);
float ShadowCalculation(vec4 fragPosLightSpace);

float bias;

void main() {

    vec3 norm = normalize(Normal);
    vec3 viewDirection = normalize(viewPosition - FragmentPosition);
	
	//global lighting
	vec3 result = CalcGlobalLight(globalLight, norm, viewDirection);

	float shadow = 0.0;
		shadow = shadows ? ShadowCalculation(FragPosLightSpace) : 0.0;
	shadow *= 0.7;	//make shadow brighter

    result = (globalLight.ambient + (1.0 - shadow) * (globalLight.diffuse + globalLight.specular)) * result;    

	color = vec4(result, 1.0);

	if (celShading) {	//cel Shading
		float intensity = dot(normalize(globalLight.position), Normal);
		vec4 color2;
		if (intensity > 0.9)
			color2 = vec4(1.0, 1.0, 1.0, 1.0);
		else if (intensity > 0.6)
			color2 = vec4(0.8, 0.8, 0.8, 1.0);
		else if (intensity > 0.1)
			color2 = vec4(0.6, 0.6, 0.6, 1.0);
		else if (intensity > -0.40)
			color2 = vec4(0.4, 0.4, 0.4, 1.0);
		else
			color2 = vec4(0.2, 0.2, 0.2, 1.0);

		color2 += vec4(material.ambient / 3, 1);
		color *= color2;
	}
}


vec3 CalcGlobalLight(GlobalLight light, vec3 normal, vec3 viewDirection) {

	//calculate the direction of the light
	vec3 lightDirection = normalize(light.position - FragmentPosition);  
	//get the angle of the light
	float diff = max(dot(normal, lightDirection), 0.0f);

	//for shadows
	bias = max(0.001 * (1.0 - dot(normal, lightDirection)), 0.0009);

    //specular shading
	float spec = 1.0;
	if (blinn) {
		vec3 halfwayDirection = normalize(lightDirection + viewDirection);  
		spec = pow(max(dot(normal, halfwayDirection), 0.0), material.shininess * 2);
    }
    else {
		vec3 reflectDirection = reflect(-lightDirection, normal);
		spec = pow(max(dot(viewDirection, reflectDirection), 0.0), material.shininess);
    }

	vec2 tCoords = TextureCoords;
	//if (celShading) {
	//	tCoords = vec2(0, 0);
	//	if (vec3(texture(material.diffuseTexture, tCoords)) == vec3(0,0,0))	//color is balck-> use original texCoords
	//		tCoords = TextureCoords;
	//}

	vec3 ambient = light.ambient * vec3(texture(material.diffuseTexture, tCoords)) * material.ambient;
	vec3 diffuse = light.diffuse * diff * vec3(texture(material.diffuseTexture, tCoords)) * material.diffuse;  
	vec3 specular = light.specular * spec * vec3(texture(material.diffuseTexture, tCoords)) * material.specular;

	return (ambient + diffuse + specular);
}

float ShadowCalculation(vec4 fragPosLightSpace) {
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // Transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // Get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r;
    // Get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // Check whether current frag pos is in shadow

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x) {
		for(int y = -1; y <= 1; ++y) {
			float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
			shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;
		}    
	}
	shadow /= 9.0;

	if(projCoords.z > 1.0)
        shadow = 0.0;

    return shadow;
}